# @summary Class responsible for installing rke2
class rke2::install {
  case $rke2::installation_mode {
    'script': {
      archive { '/tmp/rke2_install.sh':
        ensure           => present,
        filename         => '/tmp/rke2_install.sh',
        source           => 'https://get.rke2.io',
        creates          => '/tmp/rke2_install.sh',
        download_options => ['-s'],
        cleanup          => false,
      }

      file { '/tmp/rke2_install.sh':
        ensure  => file,
        mode    => '0744',
        require => [
          Archive['/tmp/rke2_install.sh'],
        ],
      }

      $environment = file('/tmp/rke2_environment').split("\n")

      # debug print the environment variable
      exec { 'print_environment':
        command     => 'printenv',
        path        => '/usr/bin:/usr/sbin:/bin:/sbin',
        environment => $environment,
        logoutput   => true,
        require     => [
          File['/tmp/rke2_install.sh'],
          File_line['rke2_node_type'],
          File_line['rke2_version'],
          File_line['rke2_channel'],
        ],
      }

      exec { '/tmp/rke2_install.sh':
        command     => '/tmp/rke2_install.sh',
        subscribe   => [
          File['/tmp/rke2_install.sh'],
        ],
        require     => [
          File_line['rke2_node_type'],
          File_line['rke2_version'],
          File_line['rke2_channel'],
        ],
        unless      => ['test -f /usr/local/bin/rke2'],
        refreshonly => true,
      }

      service { 'rke2':
        ensure     => running,
        name       => "rke2-${rke2::node_type}",
        enable     => true,
        hasrestart => true,
        hasstatus  => true,
        subscribe  => [
          File['/etc/rancher/rke2/config.yaml.d/00-config.yaml'],
        ],
        require    => [
          Exec['/tmp/rke2_install.sh'],
        ],
      }
    }

    'binary': {
      case $facts['os']['architecture'] {
        'amd64', 'x86_64': {
          $binary_arch = 'rke2'
          $checksum_arch = 'sha256sum-amd64.txt'
        }
        'arm64': {
          $binary_arch = 'rke2-arm64'
          $checksum_arch = 'sha256sum-arm64.txt'
        }
        'armhf': {
          $binary_arch = 'rke2-armhf'
          $checksum_arch = 'sha256sum-arm.txt'
        }
        default: {
          fail('No valid architecture provided.')
        }
      }
      $rke2_url = "https://github.com/rancher/rke2/releases/download/${rke2::binary_version}/rke2.linux-${binary_arch}"
      $rke2_checksum_url = "https://github.com/rancher/rke2/releases/download/${rke2::binary_version}/${checksum_arch}"

      archive { $rke2::binary_path:
        ensure           => present,
        source           => $rke2_url,
        checksum_url     => $rke2_checksum_url,
        checksum_type    => 'sha256',
        cleanup          => false,
        creates          => $rke2::binary_path,
        download_options => '-S',
        require          => [
          File['/etc/rancher/rke2/config.yaml.d/00-config.yaml'],
        ],
      }

      file { $rke2::binary_path:
        ensure  => file,
        mode    => '0755',
        require => [
          Archive[$rke2::binary_path],
        ],
      }
    }

    default: {
      fail('No valid installation mode provided.')
    }
  }
}
